==> agena-0.33.2-src.tar.gz: agena-0.33.2/src/stats.c <==
/*
** $Id: lstatlib.c,v 1.0.1 by Alexander Walz - initiated June 19, 2007
** Statistics library
** See Copyright Notice in agena.h
*/


#include <stdlib.h>
#include <math.h>
#include <string.h>

/* the following package ini declarations must be included after `#include <` and before `include #` ! */

#define stats_c
#define LUA_LIB

#include "agena.h"
#include "agnxlib.h"
#include "agenalib.h"


#if !(defined(LUA_DOS) || defined(__OS2__) || defined(LUA_ANSI))
#define AGENA_STATSLIBNAME "stats"
LUALIB_API int (luaopen_stats) (lua_State *L);
#endif


/* median of all values in an _already sorted set_, June 18, 2007; tweaked Jan 13, 2008;
   faster than an Agena implementation; patched 0.25.4, August 01, 2009; patched 0.26.1, 11.08.2009 */

static int tbl_median (lua_State *L) {
  int n;
  div_t middle;
  switch (lua_type(L, 1)) {
    case LUA_TTABLE: {
      n = agn_size(L, 1);
      middle = div(n, 2);
      if (middle.rem != 0)
        lua_rawgeti(L, 1, middle.quot+1);
      else
        lua_pushnumber(L,
          (agn_getinumber(L, 1, middle.quot) + agn_getinumber(L, 1, middle.quot+1)) / 2);
      break;
    }
    case LUA_TSEQ: {
      n = agn_seqsize(L, 1);
      if (n == 0) {
        lua_pushnumber(L, 0);
        return 1;
      }
      middle = div(n, 2);
      if (middle.rem != 0)
        lua_seqgeti(L, 1, middle.quot+1);
      else
        lua_pushnumber(L,
          (lua_seqgetinumber(L, 1, middle.quot) + lua_seqgetinumber(L, 1, middle.quot+1)) / 2);
      break;
    }
    default:
      luaL_error(L, "Error in " LUA_QS ": table or sequence expected.", "stats.median");
  }
  return 1;
}


/* tbl_minmax: returns the smallest and largest value in a list. If the option
   'sorted' is given, the list is not traversed, instead the first and the last entry
   in the list is returned.
   If the list is empty or has only one element, fail is returned. June 20, 2007;
   modified August 01, 2009, 0.25.4; modified 0.26.1, 11.08.2009 */

static int tbl_minmax (lua_State *L) {
  int i, n, type;
  lua_Number min, max, number;
  type = lua_type(L, 1);
  luaL_argcheck(L, type == LUA_TTABLE || type == LUA_TSEQ, 1, "table or sequence expected");
  const char *option = luaL_optstring(L, 2, "default");
  n = agn_nops(L, 1);
  if (n < 2) {
    lua_pushfail(L);
    return 1;
  }
  switch (type) {
    case LUA_TTABLE: {
      lua_newtable(L);
      if (strcmp(option, "sorted") == 0) {
        lua_pushinteger(L, 1);
        lua_rawgeti(L, 1, 1);
        lua_rawset(L, -3);
        lua_pushinteger(L, 2);
        lua_rawgeti(L, 1, n);
        lua_rawset(L, -3);
        return 1;
      }
      min = agn_getinumber(L, 1, 1);
      max = min;
      for (i=2; i<=n; i++) {
        number = agn_getinumber(L, 1, i);
        if (number < min)
          min = number;
        if (number > max)
          max = number;
      }
      /* FIXME: optimize this */
      lua_pushinteger(L, 1);
      lua_pushnumber(L, min);
      lua_rawset(L, -3);
      lua_pushinteger(L, 2);
      lua_pushnumber(L, max);
      lua_rawset(L, -3);
      break;
    }
    case LUA_TSEQ: {
      agn_createseq(L, 2);
      if (strcmp(option, "sorted") == 0) {
        lua_seqgeti(L, 1, 1);
        lua_seqseti(L, -2, 1);
        lua_seqgeti(L, 1, n);
        lua_seqseti(L, -2, 2);
        return 1;
      }
      min = lua_seqgetinumber(L, 1, 1);
      max = min;
      for (i=2; i<=n; i++) {
        number = lua_seqgetinumber(L, 1, i);
        if (number < min)
          min = number;
        if (number > max)
          max = number;
      }
      /* FIXME: optimize this */
      lua_pushnumber(L, min);
      lua_seqinsert(L, -2);
      lua_pushnumber(L, max);
      lua_seqinsert(L, -2);
      break;
    }
    default: lua_assert(0);
  }
  return 1;
}


static const luaL_Reg statslib[] = {
  {"median", tbl_median},                 /* added on June 19, 2007 */
  {"minmax", tbl_minmax},                 /* added on June 20, 2007 */
  {NULL, NULL}
};


static void createmetatable (lua_State *L) {
  lua_createtable(L, 0, 1);  /* create metatable for strings */
  lua_pushliteral(L, "");  /* dummy string */
  lua_pushvalue(L, -2);
  lua_setmetatable(L, -2);  /* set string metatable */
  agn_poptop(L);  /* pop dummy string */
  lua_pushvalue(L, -2);  /* string library... */
  lua_setfield(L, -2, "__index");  /* ...is the __index metamethod */
  agn_poptop(L);  /* pop metatable */
}


/*
** Open stats library
*/
LUALIB_API int luaopen_stats (lua_State *L) {
  luaL_register(L, AGENA_STATSLIBNAME, statslib);
  createmetatable(L);
  return 1;
}

/* ====================================================================== */

==> agena-1.4.0-src.tar.gz: agena-1.4.0/src/stats.c <==
/*
** $Id: lstatlib.c,v 1.0.1 by Alexander Walz - initiated June 19, 2007
** Statistics library
** See Copyright Notice in agena.h
*/


#include <stdlib.h>
#include <math.h>
#include <string.h>

/* the following package ini declarations must be included after `#include <` and before `include #` ! */

#define stats_c
#define LUA_LIB

#include "agena.h"
#include "agnxlib.h"
#include "agenalib.h"


#if !(defined(LUA_DOS) || defined(__OS2__) || defined(LUA_ANSI))
#define AGENA_STATSLIBNAME "stats"
LUALIB_API int (luaopen_stats) (lua_State *L);
#endif


/* median of all values in an _already sorted table array or sequence_, June 18, 2007; tweaked Jan 13, 2008;
   faster than an Agena implementation; patched 0.25.4, August 01, 2009; patched 0.26.1, 11.08.2009 */

static int tbl_median (lua_State *L) {
  int n;
  div_t middle;
  switch (lua_type(L, 1)) {
    case LUA_TTABLE: {
      n = agn_size(L, 1);
      middle = div(n, 2);
      if (middle.rem != 0)
        lua_rawgeti(L, 1, middle.quot+1);
      else
        lua_pushnumber(L,
          (agn_getinumber(L, 1, middle.quot) + agn_getinumber(L, 1, middle.quot+1)) / 2);
      break;
    }
    case LUA_TSEQ: {
      n = agn_seqsize(L, 1);
      if (n == 0) {
        lua_pushfail(L);
        return 1;
      }
      middle = div(n, 2);
      if (middle.rem != 0)
        lua_seqgeti(L, 1, middle.quot+1);
      else
        lua_pushnumber(L,
          (lua_seqgetinumber(L, 1, middle.quot) + lua_seqgetinumber(L, 1, middle.quot+1)) / 2);
      break;
    }
    default:
      luaL_error(L, "Error in " LUA_QS ": table or sequence expected, got.", "stats.median",
        lua_typename(L, lua_type(L, 1)));
  }
  return 1;
}


/* tbl_minmax: returns the smallest and largest value in a list. If the option
   'sorted' is given, the list is not traversed, instead the first and the last entry
   in the list is returned.
   If the list is empty or has only one element, fail is returned. June 20, 2007;
   modified August 01, 2009, 0.25.4; modified 0.26.1, 11.08.2009, extended 1.3.3, 30.01.2011 */

static int tbl_minmax (lua_State *L) {
  int i, n, type;
  lua_Number min, max, number;
  type = lua_type(L, 1);
  luaL_typecheck(L, type == LUA_TTABLE || type == LUA_TSEQ, 1, "table or sequence expected", type);
  const char *option = luaL_optstring(L, 2, "default");
  n = agn_nops(L, 1);
  if (n < 2) {
    lua_pushfail(L);
    return 1;
  }
  switch (type) {
    case LUA_TTABLE: {
      lua_newtable(L);
      if (strcmp(option, "sorted") == 0) {
        lua_pushinteger(L, 1);
        lua_rawgeti(L, 1, 1);
        lua_rawset(L, -3);
        lua_pushinteger(L, 2);
        lua_rawgeti(L, 1, n);
        lua_rawset(L, -3);
        return 1;
      }
      lua_pushnil(L);
      max = min = 0;
      while (lua_next(L, 1) != 0) {
        if (lua_isnumber(L, -1)) {
          number = agn_tonumber(L, -1);
          if (number < min)
            min = number;
          if (number > max)
            max = number;
        }
        agn_poptop(L);  /* pop value */
      }
      lua_rawsetinumber(L, -1, 1, min);
      lua_rawsetinumber(L, -1, 2, max);
      break;
    }
    case LUA_TSEQ: {
      agn_createseq(L, 2);
      if (strcmp(option, "sorted") == 0) {
        lua_seqgeti(L, 1, 1);
        lua_seqseti(L, -2, 1);
        lua_seqgeti(L, 1, n);
        lua_seqseti(L, -2, 2);
        return 1;
      }
      min = lua_seqgetinumber(L, 1, 1);
      max = min;
      for (i=2; i<=n; i++) {
        number = lua_seqgetinumber(L, 1, i);
        if (number < min)
          min = number;
        if (number > max)
          max = number;
      }
      /* FIXME: optimize this */
      lua_pushnumber(L, min);
      lua_seqinsert(L, -2);
      lua_pushnumber(L, max);
      lua_seqinsert(L, -2);
      break;
    }
    default: lua_assert(0);
  }
  return 1;
}


static const luaL_Reg statslib[] = {
  {"median", tbl_median},                 /* added on June 19, 2007 */
  {"minmax", tbl_minmax},                 /* added on June 20, 2007 */
  {NULL, NULL}
};


static void createmetatable (lua_State *L) {
  lua_createtable(L, 0, 1);  /* create metatable for strings */
  lua_pushliteral(L, "");  /* dummy string */
  lua_pushvalue(L, -2);
  lua_setmetatable(L, -2);  /* set string metatable */
  agn_poptop(L);  /* pop dummy string */
  lua_pushvalue(L, -2);  /* string library... */
  lua_setfield(L, -2, "__index");  /* ...is the __index metamethod */
  agn_poptop(L);  /* pop metatable */
}


/*
** Open stats library
*/
LUALIB_API int luaopen_stats (lua_State *L) {
  luaL_register(L, AGENA_STATSLIBNAME, statslib);
  createmetatable(L);
  return 1;
}

/* ====================================================================== */
